package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.AudioStream;
import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.SubtitleStream;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

/**
 * A converter solely for the purpose of converting video streams of MKV files into h264
 */
public class MkvH265ReducedConverter extends AbstractConverter {

    /**
     * Initializes variables used by the abstract converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     */
    public MkvH265ReducedConverter(String ffprobePath, String ffmpegPath) {
        super("mkv");
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
    }

    @Override
    public String[] getValidFormats() {
        return new String[]{"mkv"};
    }

    @Override
    public String[] generateConversionCommand(String executable, File file, List<StreamObject> streams,
                                              String outFile) {
        List<String> command = new ArrayList<>();
        command.add(executable);
        command.add("-hwaccel");
        command.add("cuda");
        command.add("-hwaccel_output_format");
        command.add("cuda");
        command.add("-i");
        command.add(file.getName());

        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        // Map video if present
        if (!filterStreamsByType(streams, VideoStream.class).isEmpty()) {
            command.add("-map");
            command.add("0:v");
            command.add("-codec:v");
            command.add("hevc_nvenc");
            command.add("-crf");
            command.add("28");
            command.add("-preset");
            command.add("slow");
            command.add("-tag:v");
            command.add("hvc1");
            command.add("-movflags");
            command.add("+faststart");
        }

        // Map audio if present
        if (!filterStreamsByType(streams, AudioStream.class).isEmpty()) {
            command.add("-map");
            command.add("0:a");
        }

        // Map subtitles if present
        if (hasInternalStreams(streams)) {
            command.add("-map");
            command.add("0:s");
            command.add("-c:s");
            command.add("copy");
        }

        command.add("-map_metadata");
        command.add("0");
        command.add("-movflags");
        command.add("use_metadata_tags");
        command.add("-f");
        command.add("matroska");

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    /**
     * Checks whether the processed file has any internal subtitle streams
     *
     * @param streams <p>All parsed streams for the video file</p>
     * @return <p>True if the file has at least one internal subtitle stream</p>
     */
    private boolean hasInternalStreams(List<StreamObject> streams) {
        for (StreamObject subtitleStream : filterStreamsByType(streams, SubtitleStream.class)) {
            if (((SubtitleStream) subtitleStream).isInternalSubtitle()) {
                return true;
            }
        }

        return false;
    }

}
